; ---------------
; -- FUNCTIONS --
; ---------------


; -- MAKE-POP-URL --

make-pop-url: func [
	"build a pop account url"
	pop-username [string!] "username"
	pop-password [string!] "password"
	pop-mail-server [string!] "mail server"
][
	pop-url: rejoin [ "pop://" pop-username ":" pop-password "@" pop-mail-server ]
	make url! pop-url
]


; -- BREAKDOWN-SUBJECT --

breakdown-subject: func [
	"breakdown an e-mail subject field into its parts"
	msg [object!] "e-mail message"
][
	subject-info: msg/subject
	subject-parts: copy []
	foreach part parse/all subject-info ":" [
		append subject-parts part
	]
]


; -- BREAKDOWN-CONTENT --

breakdown-content: func [
	"breakdown an e-mail content field into its parts"
	msg [object!] "e-mail message"
][
	article-info: msg/content
	end-of-paragraph: rejoin [{.} newline]
	replace/all article-info end-of-paragraph {.~}
	content-parts: copy []
	foreach part parse/all article-info {~} [ append content-parts trim/lines part ]
]


; -- BREAKDOWN-EMAIL --

breakdown-email: func [
	"breakdown an e-mail message object and convert it to an article object"
	msg [object!] "e-mail message"
][
	subject-pieces: breakdown-subject msg
	content-pieces: breakdown-content msg
	
	make object! [
		headline: subject-pieces/1
		subheadline: subject-pieces/2
		body: content-pieces
	]
]


; -- TIME-IN-DIGITS --

time-in-digits: func [
	"convert the date and time from 'now' into a string of digits"
	sun-dial [date!] "the current date and time from 'now'"
][
	year: to-string sun-dial/year
	month: to-string sun-dial/month
	if (length? month) < 2 [insert month "0"]
	day: to-string sun-dial/day
	if (length? day) < 2 [insert day "0"]

	current-time: sun-dial/time
	hour: to-string current-time/hour
	if (length? hour) < 2 [insert hour "0"]
	minutes: to-string current-time/minute
	if (length? minutes) < 2 [insert minutes "0"]
	seconds: to-string current-time/second
	if (length? seconds) < 2 [insert seconds "0"]
	
	rejoin [year month day hour minutes seconds]
]

; -- MAKE-COMMENT --

make-comment: func [cgi-input now-digits][
	make object! [
		heading: cgi-input/heading
		body: cgi-input/body
		name: cgi-input/name
		email: cgi-input/email
		comment-post-date: now
		reference-number: now-digits
	]
]
