#!rebol -cs

REBOL [ 
   Title:   "News Site" 
   Date:    17-September-2000 
   Version: 2.5 
   File:    %news-site.cgi 
   Author:  "Ryan C. Christiansen" 
   Email:   norsepower@uswest.net 
   Rights:  "Copyright (C) Ryan C. Christiansen 2000" 
   Purpose: {
   
      INPUT:

      A CGI executable script which checks a POP e-mail account for new mail, verifies the e-mail is from a qualified contributor (or deletes the e-mail), converts the contents of the e-mail object (as understood by REBOL) into a "make object!" expression saved to a file. These "make object!" expressions are set up as news articles, including values for headlines, subheadlines, bylines, titles, body text in paragraphs, reference numbers, file names, and approval status. The script also creates a comments directory for each new article.

      OUTPUT:

      A CGI executable script which dynamically creates the front page of a news-oriented web site. Currently prints the following as text/html output...
      -html head and meta information.
	  -stylesheet information.
	  -banner information.
	  -news headlines, bylines, articles, and article reference numbers.
	  -a link to execute displaycomments.cgi, which renders the article again, comments already made on the article, and a form for posting a new comment.
      -upon execution, news-site.cgi updates an .rss (Rich Site Summary) XML file for the site.
   } 

   Comment: {
   
      The "make object!" expressions created by this script are used to create text/html output which is a news-oriented Web page. The style of the text/html output is dependent upon a cascading style sheet which is loaded upon execution. Your browser must support cascading style sheets in order for the script to display the articles properly by default.

   } 

   History: [ 

	1.0 [	13-May-2000
		{Made scoop.cgi a public release}
		"Ryan"
	]

	2.0 [	22-June-2000
		{Renamed to News Site. New features include:
			-articles are now saved as object expressions instead of as html-formatted text files. Object expressions including the following variables:
				'headline
				'subheadline
				'article-post-date
				'byline
				'title
				'body (with each paragraph as a separate string in a series of strings)
				'reference-number
				'obj-file-name
				'approval
			-the resultant text/html output is now entirely dynamic. New features include:
				-alternating <DIV> styles
				-dynamic banner creation
				-dynamic html and meta creation
		}
		
	2.5 [	17-September-2000
		{New features include:
            -all configuration items now reside in a file called configuration.r
            -all re-usable functions now reside in a file called functions.r
            -a commenting system powered by displaycomments.cgi has been added
            -upon execution, news-site.cgi now updates a .rss (Rich Site Summary) XML file for the site

		 Known problems:
		    -parse problem with tilde "~" characters when parsing e-mail message body content for conversion to objects.
            -most recent articles are expected to show up in the tail end of a directory because of the file-naming scheme. This has proven true on all test systems with Windows NT and BeOS, but when used live on a Solaris machine the articles appeared out of order.
		    
		}
		"Ryan"
	]
   ]
]


; --------------------------------
; -- LOADING DATA AND FUNCTIONS --
; --------------------------------

do load %functions.r
do load %configuration.r

; --------------------------------------------------
; -- GATHERING NEW INPUT FROM POP E-MAIL ACCOUNTS --
; --------------------------------------------------


pop-account: make-pop-url pop-username pop-password pop-mail-server
mailbox: open pop-account

while [not tail? mailbox] [

	msg: import-email first mailbox

	for x 1 reporters/total-reporters 1 [
		email-path: reform [rejoin ["reporters/reporter" x "/email-address"] ]
		reporter-email: load email-path
		either find msg/from/1 reporter-email [

			now-digits: time-in-digits now

			article: breakdown-email msg

			byline-path: reform [rejoin ["reporters/reporter" x "/full-name"] ]
			reporter-byline: load byline-path
			article: make article [byline: reporter-byline]

			title-path: reform [rejoin ["reporters/reporter" x "/person-title"] ]
			reporter-title: load title-path
			article: make article [title: reporter-title]

			article: make article [reference-number: now-digits]

			name-for-object: rejoin [{%} now-digits {.r}]
			article: make article [obj-file-name: name-for-object]

			article: make article [approval: false]

			posted-on: now
			article: make article [article-post-date: posted-on]

			save-object: reform ["save" rejoin [{%} unapproved-article-folder now-digits {.r}] "article"]

			do save-object

			make-comments-directory: reform ["make-dir" rejoin [{%} unapproved-comments-folder now-digits {/}]]
			
			do make-comments-directory

			wait 1

			][
			
			ignore: []
			clear ignore
		]
	]

	remove mailbox

]

close mailbox


; ---------------------------------------------------------------
; -- READING FILES AND SENDING TEXT/HTML OUTPUT TO THE BROWSER --
; ---------------------------------------------------------------


read-directory: reform [ "news-directory: read" rejoin [{%} approved-article-folder] ]
do read-directory

news-directory: tail news-directory

file-list: copy []
for file-grab 1 number-of-headlines 1 [
	news-directory: back news-directory
	file-name: first news-directory
	append file-list file-name
]

print "Content-Type: text/html^/"

print rejoin [	{<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">} newline newline
				{<HTML>} newline newline
				{<HEAD>} newline newline
				{<TITLE>} page-title {</TITLE>} newline newline
				{<META NAME="Keywords" CONTENT="} meta-keywords {">} newline newline
				{</HEAD>} newline newline
				]

stylesheet: read headline-stylesheet
print rejoin [stylesheet newline newline]

print rejoin [{<BODY>} newline newline]

print rejoin [	{<TABLE width="} page-width {">} newline
				{<TR>} newline
				{<TD>} newline
				]

print rejoin [	{<DIV class="banner">} newline
				{<HR>} newline
				{<font class="banner">} publication-name {</font>} newline
				{<HR>} newline
				{<font class="copyright">} publication-name {<small><SUP>TM</SUP></small> &nbsp &nbsp} copyright-info {</font>} newline
				{<HR>} newline
				{</DIV>} newline newline
				]

print rejoin [	{<TABLE width="} headlines-width {">} newline
				{<TR>} newline
				{<TD>} newline
				]

div-style: 0

foreach file-name file-list [
	read-article: reform ["article-content: do load" rejoin [{%} approved-article-folder file-name] ]
	do read-article
	article-temp: make article-content []

	div-style: div-style + 1
	either even? div-style [
		print rejoin [{<DIV class="article2">} newline newline]
		] [
		print rejoin [{<DIV class="article">} newline newline]
		]

	print rejoin [	{<P class="headline">}
					{<font class="headline">} article-temp/headline {</font>}
					{<BR>} newline
					]

	print rejoin [	{<font class="subheadline">} article-temp/subheadline {</font>}
					{<BR>} newline
					]

	print rejoin [{<font class="articlepostdate">} {posted on } article-temp/article-post-date {</font>} newline newline]

	print rejoin [	{<P class="byline">}
					{<font class="byline">} article-temp/byline {</font>}
					{<BR>} newline
					]

	print rejoin [{<font class="title">} article-temp/title {</font>} newline newline]

	pieces: length? article-temp/body
	
	for y 1 pieces 1 [
		piece-path: reform [rejoin ["article-temp/body/" y] ]
		body-piece: load piece-path

		print rejoin [	{<P class="body">}
						{<font class="body">} body-piece {</font>} newline newline
						]
	]

	print rejoin [	{<BR>} newline
			{<font class="comments">} {<A HREF="http://www.beosjournal.com/cgi-bin/displaycomments.cgi?commentsreference=} article-temp/reference-number {" target="_blank">}  {Comments} {</A>} {</font>} newline newline
			]

	print rejoin [	{<P class="referencenumber">}
					{<font class="referencenumber">} {REF: } article-temp/reference-number {</font>}
					{<BR>} newline
					]

	print rejoin [{</DIV>} newline newline]
]

print rejoin [{</TABLE>} newline newline]
print rejoin [{</TABLE>} newline newline]

print rejoin [	{</BODY>} newline newline
				{</HTML>}
				]


; ------------------------------------------
; -- READING FILES AND CREATING .RSS FILE --
; ------------------------------------------


read-directory: reform [ "news-directory: read" rejoin [{%} approved-article-folder] ]
do read-directory

news-directory: tail news-directory

file-list: copy []
for file-grab 1 number-of-headlines 1 [
	news-directory: back news-directory
	file-name: first news-directory
	append file-list file-name
]

rss-file: make string! 10000

rss-header: rejoin [	{<?xml version="1.0"?>} newline newline
			{<!DOCTYPE rss PUBLIC "-//Netscape Communications//DTD RSS 0.91//EN" "http://my.netscape.com/publish/formats/rss-0.91.dtd"><rss version="0.91">} newline newline
			{<channel>} newline newline
			{<title>} publication-name {</title>} newline
			{<link>} publication-link {</link>} newline
			{<description>} publication-description {</description>} newline
			{<language>} language {</language>} newline newline
			]

append rss-file rss-header

foreach file-name file-list [
	read-article: reform ["article-content: do load" rejoin [{%} approved-article-folder file-name] ]
	do read-article
	article-temp: make article-content []

	item: copy []
	item-link: copy []

	item-link: rejoin [{http://www.beosjournal.com/cgi-bin/displaycomments.cgi?commentsreference=} article-temp/reference-number]

	item: rejoin [	{<item>} newline
			tab {<title>} article-temp/headline {</title>} newline
			tab {<link>} item-link {</link>} newline
			tab {<description>} article-temp/body/1 {</description>} newline
			{</item>} newline newline
			]

	append rss-file item
]

rss-footer: rejoin [	{</channel>} newline
			{</rss>}
			]

append rss-file rss-footer

write-rss: reform ["write" rejoin [{%} headlines-directory {headlines.rss}] "rss-file"]

do write-rss
